function M = getM(p, x1, x2),

% GETM return Zernikie (OSA) terms 
%
%   M = getM(p,phi,r),
%
% (x1,x2) = (r,phi) or (u,v)
%
% matrix where N rows are terms for each (r_i, phi_i) (i = 1 ... N)
% and the columns are for the M terms (j = 1 ... M) 
%
% where 
%       p   = psurf object
%       r   = column vector of rs  
%       phi = row vector of phi
%

% added matrix return capability

if (strcmp(p.coord,'cartesian')),
    [phi,r] = cart2pol(x1,x2);
    r       = r/p.aperture;
else
    r   = x1/p.aperture;
    phi = x2;
end 

r   = r(:);
phi = phi(:);

% normalized coefficients
M   = bhwRepMtx(r, phi, size(p.terms,1));

return

