function [z, varargout] = getMdv(p, x1, x2, strf),

% GETMDV return Zernikie (OSA) terms 
%
%   z = getMdv(p, x1, x2, strf),
%
% matrix where N rows are terms for each (r_i, phi_i) (i = 1 ... N)
% and the columns are for the M terms (j = 1 ... M) 
%
% where 
%       p   = psurf object
%       r   = column vector of rs  
%       phi = row vector of phi
%
% 26/1/06           -- changed to a switch/case format 
% modified 24/10/05 -- aperture re-scaling added.

nAp = p.aperture;

% added matrix return capability
if (strcmp(p.coord,'cartesian')),
    [phi,r] = cart2pol(x1,x2);
    r       = r/nAp;
else
    r   = x1/nAp;
    phi = x2;
end 

p   = get(p,'terms');

r = r(:); phi = phi(:);
switch(strf)
    case { 'rho','dzdr'}
        z       = znkBasisOSADVec(p, r, phi, 'rho');
        z       = z*(1/nAp);
        return
    case { 'phi','dzdt' }
        z       = znkBasisOSADVec(p, r, phi, 'phi');
        return
    case { 'u','v','dzdx','dzdy' }
        dDdr    = znkBasisOSADVec(p, r, phi, 'rho')*(1/nAp);
        dDdphi  = znkBasisOSADVec(p, r, phi, 'phi');
        n       = size(dDdr,2);
        phi     = phi(:,ones([ 1 n ] ));
        r       = r(:,ones([ 1 n ] ));
        z       = dpol2dcar(phi, r*nAp, dDdphi, dDdr, strf);
        return
    case { 'u-v' },
        dDdr    = znkBasisOSADVec(p, r, phi, 'rho')*(1/nAp);
        dDdphi  = znkBasisOSADVec(p, r, phi, 'phi');
        n       = size(dDdr,2);
        phi     = phi(:,ones([ 1 n ] ));
        r       = r(:,ones([ 1 n ] ));
        z       = dpol2dcar(phi, r*nAp, dDdphi, dDdr, 'u');
        varargout{1} = dpol2dcar(phi, r*nAp, dDdphi, dDdr, 'v');
        return
    otherwise 
        error([ 'Unknown flag (' strf ')' ]);
end 
    
return

