function h = surf(z, varargin)

% SURF a 2D plotting function for Zernike polynomials 
% 
% SURF(z, ...).  Additional parameters will be passed 
% as dewscxribed in help for SURF.  An additional 
% parameter be specified i.e., 'grid', changes the 
% gridding used (128 is default). 
% 
% EXAMPLE,
%
%   % Zernike    0.5*Znk(0) + Znk(1)   
%   b = znk(1); c = [ 0.5 1 0 ];
%   b = set(b, 'coeff', c');
%   surf(b);  % surf(b, 'grid', 64) % lower gridding

[ngrid, nargs]   = getProperty('grid', varargin, 128);

nAp     = get(z, 'aperture');
x       = linspace(-nAp, nAp, ngrid);
[x, y]  = meshgrid(x, x);
r       = sqrt(x.^2 + (x').^2);
z       = set(z, 'coord', 'cartesian');
z       = get(z, x, y);

z(r > nAp) = nan;

if (~isempty(nargs)),
    h  = surf(x, y, z, nargs{:});
else
    h  = surf(x, y, z);
end
   

return