function y = bhwRepMtx(r, phi, N),

% BHWREPMTX Bhatia-Wolf representation matrix 
%
% Bhatia-Wolf matrix with columns (p = 1 ... N) over the
% polynomial-ordering parameter, and a row for each (r,phi) pair.  
%
%   y = bhwRepMtx(r, phi, p),
%
% where 
%       y   = Bhatia-Wolf matrix 
%       r   = radial distance (<= 1)
%       phi = angle in radians 
%       N   = maximum polynomial order (1<= p <= N)
%
% see BHWTERM for explanation of the Bhatia-Wolf polynomials        
%
% implements Bhatia=Wolf polynomials as described in : 
%
% Modeling of Corneal Surfaces with Radial Polynomials. Iskander, DR;
% Morelande, MR; Collins MJ; Davis, B. IEEE Trams. Biomed. Eng. 49(4) April
% 2002. 

r   = r(:);
phi = phi(:);

for p = 1:N,
    y(:,p) = bhwTerm(r, phi, p);
end 

return 
