function A = bsp1dcoeff(i, p, kt)

% BSP1DCOEFF  return the information for the
% particular recursion level - given the knot
% sequence, the control point and order
% 
%   e = bsp1dcoeff(i, p, kt)
% 
% where     
%       i = control point
%       p = degree 
% 
% where k = p + 1 is the order (4th order is cubic).  
% 
% The Cox-de Boor recursion formula gives,
% 
%              (t - a1)                (b1 - t)
%  N(i,p)(t) = -------- N(i,p-1)(t) +  -------- N(i+1,p-1)(t)
%                 a2                      b2
% 
% where, 
%
%   a1 = kt(i)
%   a2 = kt(i+p) - kt(i)
%   b1 = kt(i+p+1)
%   b2 = kt(i+p+1) - kt(i+1)
% 
% convention is that 0/0 is 0 
%
% 30/10/06 move to mathworld consistent nimenclature 

if p == 0
   A = 0;
   return
end

a1 = kt(i+1);
a2 = kt(i+p+1) - kt(i+1);
b1 = kt(i+p+1+1);
b2 = kt(i+p+1+1) - kt(i+1+1);
 
A = [ a1 a2 b1 b2 ];
return
