function M = znkQuickBasis(r, phi, N, strf),

% ZNKQUICKBASIS Return quick basis M matrix. 
%
% ZNKQUICKBASIS(r, phi, N, strf). The variable N is the largest index,
% where j = 0, ..., N.  The 1st derivative { 'dzdr', '  g_dzdt' } or the 
% depth { 'z' }.
% 
% ZNKQUICKBASIS(r, phi, N).  Return depth by default.
%
% NOTES
%   28/3/2007 : currently stores the 1st 66 terms 

persistent g_z g_dzdr g_dzdt;
    
if (nargin == 3),
    strf = 'z';
end

% keyboard;
% fprintf('(ZNKQUICKBASIS)\n');

switch(strf),
   case {'z'}
     if (isempty(  g_z)), 
        g_z{1}='1+0*r+0*phi';
        g_z{2}='2*(r)*sin(phi)';
        g_z{3}='2*(r)*cos(phi)';
        g_z{4}='sqrt(6)*(r^2)*sin(2*phi)';
        g_z{5}='sqrt(3)*(2*r^2-1)';
        g_z{6}='sqrt(6)*(r^2)*cos(2*phi)';
        g_z{7}='sqrt(8)*(r^3)*sin(3*phi)';
        g_z{8}='sqrt(8)*(3*r^3-2*r)*sin(phi)';
        g_z{9}='sqrt(8)*(3*r^3-2*r)*cos(phi)';
        g_z{10}='sqrt(8)*(r^3)*cos(3*phi)';
        g_z{11}='sqrt(10)*(r^4)*sin(4*phi)';
        g_z{12}='sqrt(10)*(4*r^4-3*r^2)*sin(2*phi)';
        g_z{13}='sqrt(5)*(6*r^4-6*r^2+1)';
        g_z{14}='sqrt(10)*(4*r^4-3*r^2)*cos(2*phi)';
        g_z{15}='sqrt(10)*(r^4)*cos(4*phi)';
        g_z{16}='sqrt(12)*(r^5)*sin(5*phi)';
        g_z{17}='sqrt(12)*(5*r^5-4*r^3)*sin(3*phi)';
        g_z{18}='sqrt(12)*(10*r^5-12*r^3+3*r)*sin(phi)';
        g_z{19}='sqrt(12)*(10*r^5-12*r^3+3*r)*cos(phi)';
        g_z{20}='sqrt(12)*(5*r^5-4*r^3)*cos(3*phi)';
        g_z{21}='sqrt(12)*(r^5)*cos(5*phi)';
        g_z{22}='sqrt(14)*(r^6)*sin(6*phi)';
        g_z{23}='sqrt(14)*(6*r^6-5*r^4)*sin(4*phi)';
        g_z{24}='sqrt(14)*(15*r^6-20*r^4+6*r^2)*sin(2*phi)';
        g_z{25}='sqrt(7)*(20*r^6-30*r^4+12*r^2-1)';
        g_z{26}='sqrt(14)*(15*r^6-20*r^4+6*r^2)*cos(2*phi)';
        g_z{27}='sqrt(14)*(6*r^6-5*r^4)*cos(4*phi)';
        g_z{28}='sqrt(14)*(r^6)*cos(6*phi)';
        g_z{29}='4*(r^7)*sin(7*phi)';
        g_z{30}='4*(7*r^7-6*r^5)*sin(5*phi)';
        g_z{31}='4*(21*r^7-30*r^5+10*r^3)*sin(3*phi)';
        g_z{32}='4*(35*r^7-60*r^5+30*r^3-4*r)*sin(phi)';
        g_z{33}='4*(35*r^7-60*r^5+30*r^3-4*r)*cos(phi)';
        g_z{34}='4*(21*r^7-30*r^5+10*r^3)*cos(3*phi)';
        g_z{35}='4*(7*r^7-6*r^5)*cos(5*phi)';
        g_z{36}='4*(r^7)*cos(7*phi)';
        g_z{37}='sqrt(18)*(r^8)*sin(8*phi)';
        g_z{38}='sqrt(18)*(8*r^8-7*r^6)*sin(6*phi)';
        g_z{39}='sqrt(18)*(28*r^8-42*r^6+15*r^4)*sin(4*phi)';
        g_z{40}='sqrt(18)*(56*r^8-105*r^6+60*r^4-10*r^2)*sin(2*phi)';
        g_z{41}='3*(70*r^8-140*r^6+90*r^4-20*r^2+1)';
        g_z{42}='sqrt(18)*(56*r^8-105*r^6+60*r^4-10*r^2)*cos(2*phi)';
        g_z{43}='sqrt(18)*(28*r^8-42*r^6+15*r^4)*cos(4*phi)';
        g_z{44}='sqrt(18)*(8*r^8-7*r^6)*cos(6*phi)';
        g_z{45}='sqrt(18)*(r^8)*cos(8*phi)';
        g_z{46}='sqrt(20)*(r^9)*sin(9*phi)';
        g_z{47}='sqrt(20)*(9*r^9-8*r^7)*sin(7*phi)';
        g_z{48}='sqrt(20)*(36*r^9-56*r^7+21*r^5)*sin(5*phi)';
        g_z{49}='sqrt(20)*(84*r^9-168*r^7+105*r^5-20*r^3)*sin(3*phi)';
        g_z{50}='sqrt(20)*(126*r^9-280*r^7+210*r^5-60*r^3+5*r)*sin(phi)';
        g_z{51}='sqrt(20)*(126*r^9-280*r^7+210*r^5-60*r^3+5*r)*cos(phi)';
        g_z{52}='sqrt(20)*(84*r^9-168*r^7+105*r^5-20*r^3)*cos(3*phi)';
        g_z{53}='sqrt(20)*(36*r^9-56*r^7+21*r^5)*cos(5*phi)';
        g_z{54}='sqrt(20)*(9*r^9-8*r^7)*cos(7*phi)';
        g_z{55}='sqrt(20)*(r^9)*cos(9*phi)';
        g_z{56}='sqrt(22)*(r^10)*sin(10*phi)';
        g_z{57}='sqrt(22)*(10*r^10-9*r^8)*sin(8*phi)';
        g_z{58}='sqrt(22)*(45*r^10-72*r^8+28*r^6)*sin(6*phi)';
        g_z{59}='sqrt(22)*(120*r^10-252*r^8+168*r^6-35*r^4)*sin(4*phi)';
        g_z{60}='sqrt(22)*(210*r^10-504*r^8+420*r^6-140*r^4+15*r^2)*sin(2*phi)';
        g_z{61}='sqrt(11)*(252*r^10-630*r^8+560*r^6-210*r^4+30*r^2-1)';
        g_z{62}='sqrt(22)*(210*r^10-504*r^8+420*r^6-140*r^4+15*r^2)*cos(2*phi)';
        g_z{63}='sqrt(22)*(120*r^10-252*r^8+168*r^6-35*r^4)*cos(4*phi)';
        g_z{64}='sqrt(22)*(45*r^10-72*r^8+28*r^6)*cos(6*phi)';
        g_z{65}='sqrt(22)*(10*r^10-9*r^8)*cos(8*phi)';
        g_z{66}='sqrt(22)*(r^10)*cos(10*phi)';
     end

     for k = 1:(N+1), 
         y  = vectorize(inline(g_z{k}, 'r', 'phi'));
         M(:,k) = y(r, phi);         
     end
     return
     
     
    case {'rho', 'dzdr'},  
     if (isempty(g_dzdr)),
        g_dzdr{1}='0+0*r';
        g_dzdr{2}='2*sin(phi)';
        g_dzdr{3}='2*cos(phi)';
        g_dzdr{4}='sqrt(6)*(2*r)*sin(2*phi)';
        g_dzdr{5}='sqrt(3)*(4*r)';
        g_dzdr{6}='sqrt(6)*(2*r)*cos(2*phi)';
        g_dzdr{7}='sqrt(8)*(3*r^2)*sin(3*phi)';
        g_dzdr{8}='sqrt(8)*(9*r^2-2)*sin(phi)';
        g_dzdr{9}='sqrt(8)*(9*r^2-2)*cos(phi)';
        g_dzdr{10}='sqrt(8)*(3*r^2)*cos(3*phi)';
        g_dzdr{11}='sqrt(10)*(4*r^3)*sin(4*phi)';
        g_dzdr{12}='sqrt(10)*(16*r^3-6*r)*sin(2*phi)';
        g_dzdr{13}='sqrt(5)*(24*r^3-12*r)';
        g_dzdr{14}='sqrt(10)*(16*r^3-6*r)*cos(2*phi)';
        g_dzdr{15}='sqrt(10)*(4*r^3)*cos(4*phi)';
        g_dzdr{16}='sqrt(12)*(5*r^4)*sin(5*phi)';
        g_dzdr{17}='sqrt(12)*(25*r^4-12*r^2)*sin(3*phi)';
        g_dzdr{18}='sqrt(12)*(50*r^4-36*r^2+3)*sin(phi)';
        g_dzdr{19}='sqrt(12)*(50*r^4-36*r^2+3)*cos(phi)';
        g_dzdr{20}='sqrt(12)*(25*r^4-12*r^2)*cos(3*phi)';
        g_dzdr{21}='sqrt(12)*(5*r^4)*cos(5*phi)';
        g_dzdr{22}='sqrt(14)*(6*r^5)*sin(6*phi)';
        g_dzdr{23}='sqrt(14)*(36*r^5-20*r^3)*sin(4*phi)';
        g_dzdr{24}='sqrt(14)*(90*r^5-80*r^3+12*r)*sin(2*phi)';
        g_dzdr{25}='sqrt(7)*(120*r^5-120*r^3+24*r)';
        g_dzdr{26}='sqrt(14)*(90*r^5-80*r^3+12*r)*cos(2*phi)';
        g_dzdr{27}='sqrt(14)*(36*r^5-20*r^3)*cos(4*phi)';
        g_dzdr{28}='sqrt(14)*(6*r^5)*cos(6*phi)';
        g_dzdr{29}='4*(7*r^6)*sin(7*phi)';
        g_dzdr{30}='4*(49*r^6-30*r^4)*sin(5*phi)';
        g_dzdr{31}='4*(147*r^6-150*r^4+30*r^2)*sin(3*phi)';
        g_dzdr{32}='4*(245*r^6-300*r^4+90*r^2-4)*sin(phi)';
        g_dzdr{33}='4*(245*r^6-300*r^4+90*r^2-4)*cos(phi)';
        g_dzdr{34}='4*(147*r^6-150*r^4+30*r^2)*cos(3*phi)';
        g_dzdr{35}='4*(49*r^6-30*r^4)*cos(5*phi)';
        g_dzdr{36}='4*(7*r^6)*cos(7*phi)';
        g_dzdr{37}='sqrt(18)*(8*r^7)*sin(8*phi)';
        g_dzdr{38}='sqrt(18)*(64*r^7-42*r^5)*sin(6*phi)';
        g_dzdr{39}='sqrt(18)*(224*r^7-252*r^5+60*r^3)*sin(4*phi)';
        g_dzdr{40}='sqrt(18)*(448*r^7-630*r^5+240*r^3-20*r)*sin(2*phi)';
        g_dzdr{41}='3*(560*r^7-840*r^5+360*r^3-40*r)';
        g_dzdr{42}='sqrt(18)*(448*r^7-630*r^5+240*r^3-20*r)*cos(2*phi)';
        g_dzdr{43}='sqrt(18)*(224*r^7-252*r^5+60*r^3)*cos(4*phi)';
        g_dzdr{44}='sqrt(18)*(64*r^7-42*r^5)*cos(6*phi)';
        g_dzdr{45}='sqrt(18)*(8*r^7)*cos(8*phi)';
        g_dzdr{46}='sqrt(20)*(9*r^8)*sin(9*phi)';
        g_dzdr{47}='sqrt(20)*(81*r^8-56*r^6)*sin(7*phi)';
        g_dzdr{48}='sqrt(20)*(324*r^8-392*r^6+105*r^4)*sin(5*phi)';
        g_dzdr{49}='sqrt(20)*(756*r^8-1176*r^6+525*r^4-60*r^2)*sin(3*phi)';
        g_dzdr{50}='sqrt(20)*(1134*r^8-1960*r^6+1050*r^4-180*r^2+5)*sin(phi)';
        g_dzdr{51}='sqrt(20)*(1134*r^8-1960*r^6+1050*r^4-180*r^2+5)*cos(phi)';
        g_dzdr{52}='sqrt(20)*(756*r^8-1176*r^6+525*r^4-60*r^2)*cos(3*phi)';
        g_dzdr{53}='sqrt(20)*(324*r^8-392*r^6+105*r^4)*cos(5*phi)';
        g_dzdr{54}='sqrt(20)*(81*r^8-56*r^6)*cos(7*phi)';
        g_dzdr{55}='sqrt(20)*(9*r^8)*cos(9*phi)';
        g_dzdr{56}='sqrt(22)*(10*r^9)*sin(10*phi)';
        g_dzdr{57}='sqrt(22)*(100*r^9-72*r^7)*sin(8*phi)';
        g_dzdr{58}='sqrt(22)*(450*r^9-576*r^7+168*r^5)*sin(6*phi)';
        g_dzdr{59}='sqrt(22)*(1200*r^9-2016*r^7+1008*r^5-140*r^3)*sin(4*phi)';
        g_dzdr{60}='sqrt(22)*(2100*r^9-4032*r^7+2520*r^5-560*r^3+30*r)*sin(2*phi)';
        g_dzdr{61}='sqrt(11)*(2520*r^9-5040*r^7+3360*r^5-840*r^3+60*r)';
        g_dzdr{62}='sqrt(22)*(2100*r^9-4032*r^7+2520*r^5-560*r^3+30*r)*cos(2*phi)';
        g_dzdr{63}='sqrt(22)*(1200*r^9-2016*r^7+1008*r^5-140*r^3)*cos(4*phi)';
        g_dzdr{64}='sqrt(22)*(450*r^9-576*r^7+168*r^5)*cos(6*phi)';
        g_dzdr{65}='sqrt(22)*(100*r^9-72*r^7)*cos(8*phi)';
        g_dzdr{66}='sqrt(22)*(10*r^9)*cos(10*phi)';
     end
      
     for k = 1:(N+1),
         y = vectorize(inline(g_dzdr{k},'r','phi'));
         M(:,k) = y(r, phi);         
     end
     return
              
         
    case {'phi', 'dzdt'},
      if (isempty(g_dzdt)),
        g_dzdt{1}='0+0*r';
        g_dzdt{2}='2*(r)*cos(phi)';
        g_dzdt{3}='-2*(r)*sin(phi)';
        g_dzdt{4}='2*sqrt(6)*(r^2)*cos(2*phi)';
        g_dzdt{5}='0+0*r';
        g_dzdt{6}='-2*sqrt(6)*(r^2)*sin(2*phi)';
        g_dzdt{7}='3*sqrt(8)*(r^3)*cos(3*phi)';
        g_dzdt{8}='sqrt(8)*(3*r^3-2*r)*cos(phi)';
        g_dzdt{9}='-sqrt(8)*(3*r^3-2*r)*sin(phi)';
        g_dzdt{10}='-3*sqrt(8)*(r^3)*sin(3*phi)';
        g_dzdt{11}='4*sqrt(10)*(r^4)*cos(4*phi)';
        g_dzdt{12}='2*sqrt(10)*(4*r^4-3*r^2)*cos(2*phi)';
        g_dzdt{13}='0+0*r';
        g_dzdt{14}='-2*sqrt(10)*(4*r^4-3*r^2)*sin(2*phi)';
        g_dzdt{15}='-4*sqrt(10)*(r^4)*sin(4*phi)';
        g_dzdt{16}='5*sqrt(12)*(r^5)*cos(5*phi)';
        g_dzdt{17}='3*sqrt(12)*(5*r^5-4*r^3)*cos(3*phi)';
        g_dzdt{18}='sqrt(12)*(10*r^5-12*r^3+3*r)*cos(phi)';
        g_dzdt{19}='-sqrt(12)*(10*r^5-12*r^3+3*r)*sin(phi)';
        g_dzdt{20}='-3*sqrt(12)*(5*r^5-4*r^3)*sin(3*phi)';
        g_dzdt{21}='-5*sqrt(12)*(r^5)*sin(5*phi)';
        g_dzdt{22}='6*sqrt(14)*(r^6)*cos(6*phi)';
        g_dzdt{23}='4*sqrt(14)*(6*r^6-5*r^4)*cos(4*phi)';
        g_dzdt{24}='2*sqrt(14)*(15*r^6-20*r^4+6*r^2)*cos(2*phi)';
        g_dzdt{25}='0+0*r';
        g_dzdt{26}='-2*sqrt(14)*(15*r^6-20*r^4+6*r^2)*sin(2*phi)';
        g_dzdt{27}='-4*sqrt(14)*(6*r^6-5*r^4)*sin(4*phi)';
        g_dzdt{28}='-6*sqrt(14)*(r^6)*sin(6*phi)';
        g_dzdt{29}='7*4*(r^7)*cos(7*phi)';
        g_dzdt{30}='5*4*(7*r^7-6*r^5)*cos(5*phi)';
        g_dzdt{31}='3*4*(21*r^7-30*r^5+10*r^3)*cos(3*phi)';
        g_dzdt{32}='4*(35*r^7-60*r^5+30*r^3-4*r)*cos(phi)';
        g_dzdt{33}='-4*(35*r^7-60*r^5+30*r^3-4*r)*sin(phi)';
        g_dzdt{34}='-3*4*(21*r^7-30*r^5+10*r^3)*sin(3*phi)';
        g_dzdt{35}='-5*4*(7*r^7-6*r^5)*sin(5*phi)';
        g_dzdt{36}='-7*4*(r^7)*sin(7*phi)';
        g_dzdt{37}='8*sqrt(18)*(r^8)*cos(8*phi)';
        g_dzdt{38}='6*sqrt(18)*(8*r^8-7*r^6)*cos(6*phi)';
        g_dzdt{39}='4*sqrt(18)*(28*r^8-42*r^6+15*r^4)*cos(4*phi)';
        g_dzdt{40}='2*sqrt(18)*(56*r^8-105*r^6+60*r^4-10*r^2)*cos(2*phi)';
        g_dzdt{41}='0+0*r';
        g_dzdt{42}='-2*sqrt(18)*(56*r^8-105*r^6+60*r^4-10*r^2)*sin(2*phi)';
        g_dzdt{43}='-4*sqrt(18)*(28*r^8-42*r^6+15*r^4)*sin(4*phi)';
        g_dzdt{44}='-6*sqrt(18)*(8*r^8-7*r^6)*sin(6*phi)';
        g_dzdt{45}='-8*sqrt(18)*(r^8)*sin(8*phi)';
        g_dzdt{46}='9*sqrt(20)*(r^9)*cos(9*phi)';
        g_dzdt{47}='7*sqrt(20)*(9*r^9-8*r^7)*cos(7*phi)';
        g_dzdt{48}='5*sqrt(20)*(36*r^9-56*r^7+21*r^5)*cos(5*phi)';
        g_dzdt{49}='3*sqrt(20)*(84*r^9-168*r^7+105*r^5-20*r^3)*cos(3*phi)';
        g_dzdt{50}='sqrt(20)*(126*r^9-280*r^7+210*r^5-60*r^3+5*r)*cos(phi)';
        g_dzdt{51}='-sqrt(20)*(126*r^9-280*r^7+210*r^5-60*r^3+5*r)*sin(phi)';
        g_dzdt{52}='-3*sqrt(20)*(84*r^9-168*r^7+105*r^5-20*r^3)*sin(3*phi)';
        g_dzdt{53}='-5*sqrt(20)*(36*r^9-56*r^7+21*r^5)*sin(5*phi)';
        g_dzdt{54}='-7*sqrt(20)*(9*r^9-8*r^7)*sin(7*phi)';
        g_dzdt{55}='-9*sqrt(20)*(r^9)*sin(9*phi)';
        g_dzdt{56}='10*sqrt(22)*(r^10)*cos(10*phi)';
        g_dzdt{57}='8*sqrt(22)*(10*r^10-9*r^8)*cos(8*phi)';
        g_dzdt{58}='6*sqrt(22)*(45*r^10-72*r^8+28*r^6)*cos(6*phi)';
        g_dzdt{59}='4*sqrt(22)*(120*r^10-252*r^8+168*r^6-35*r^4)*cos(4*phi)';
        g_dzdt{60}='2*sqrt(22)*(210*r^10-504*r^8+420*r^6-140*r^4+15*r^2)*cos(2*phi)';
        g_dzdt{61}='0+0*r';
        g_dzdt{62}='-2*sqrt(22)*(210*r^10-504*r^8+420*r^6-140*r^4+15*r^2)*sin(2*phi)';
        g_dzdt{63}='-4*sqrt(22)*(120*r^10-252*r^8+168*r^6-35*r^4)*sin(4*phi)';
        g_dzdt{64}='-6*sqrt(22)*(45*r^10-72*r^8+28*r^6)*sin(6*phi)';
        g_dzdt{65}='-8*sqrt(22)*(10*r^10-9*r^8)*sin(8*phi)';
        g_dzdt{66}='-10*sqrt(22)*(r^10)*sin(10*phi)';
      end

      for k = 1:(N+1),
         y = vectorize(inline(g_dzdt{k},'r','phi'));
         M(:,k) = y(r, phi);         
      end 
      return
           
      otherwise,
       error('(ZNKQUICKBASIS) Unknown invocation');   
  
end


% looks like it.
return
