function [n, m] = znkSgl2Dbl(j),

% ZNKSGL2DBL convert a single indexed Zernike to a double  
% indexed Zernike using the OSA standard.
%
% [n, m] = ZNKSGL2DBL(j). Here n is the radial order and m is the 
% (angular) frequency.  The index is the single index starting from 
% j =0.
%
% Standards for reporting the optical aberrations of the eye
% Thibos LN, Applegate RA, Schwiegerling JT, Webb R, Journal of
% Refractive Surgery; Sep/Oct 2002; 18,5 ; Health & Medical Complete pg.
% S652
%
%   j = 0.5*( n(n+2) + m )                single indexing scheme 
%   n = ceil(0.5*(-3 + sqrt(9 + 8*j)))    radial order 
%   m = 2*j - n*(n+2)                     angular frequency
%
% see Malacara 

    % j = (0:1:j)';
    n = ceil(0.5*(-3 + sqrt(9 + 8*j)));    
    m = 2*j - n.*(n+2);
